//==============================================================
//  ϱ

static const int 
	CONNECTED_RAIL_TIME=60,
	CONNECTED_RAIL_VX[]={1, 0, -1, 0},
	CONNECTED_RAIL_VY[]={0, 1, 0, -1},
	CONNECTED_RAIL_DIR[][4]={
		{4, 4, 4, 4}, 
		{4, 4, 1, 0}, {1, 4, 4, 2}, 
		{3, 2, 4, 4}, {4, 0, 3, 4}, 
		{0, 4, 2, 4}, {4, 1, 4, 3}, 
		{0, 1, 2, 3}
	};

// 
class CConnectedRailMan : public CMover {
public:
	CCell* Cell;
	int CX, CY, Dir, Time;

	CConnectedRailMan(CCell* cell, int cx, int cy, int dir)
	:	Cell(cell), CX(cx), CY(cy), Dir(dir)
	{
		X=CX+1.0f/3;
		Y=CY+1.0f/3;
		Time=CONNECTED_RAIL_TIME/2;
	}

	virtual bool Move(const CInputState* is) {
		if (Time<CONNECTED_RAIL_TIME) {
			Time++;
		} else {
			int 
				cx=CX+CONNECTED_RAIL_VX[Dir], 
				cy=CY+CONNECTED_RAIL_VY[Dir];
			if (
				cx>=0 && cx<Cell->GetXSize() &&
				cy>=0 && cy<Cell->GetYSize()
			) {
				int dir=CONNECTED_RAIL_DIR[Cell->Get(cx, cy)-'0'][Dir];
				if (dir!=4) {
					CX=cx;
					CY=cy;
					Dir=dir;
					Time=0;
				}
			}
		}

		char c=Cell->Get(CX, CY)-'0'-1;
		if (0<=c && c<=3) {
			static const int 
				rx[]={1, 0, 0, 1},
				ry[]={1, 1, 0, 0},
				rad[][4]={
					{2, 3, 4, 4}, {4, 3, 0, 4}, 
					{4, 4, 0, 1}, {2, 4, 4, 1}
				},
				vrad[][4]={
					{1, -1, 0, 0}, {0, 1, -1, 0},
					{0, 0, 1, -1}, {-1, 0, 0, 1}
				};
			float f=(rad[c][Dir]+vrad[c][Dir]*(float)Time/CONNECTED_RAIL_TIME)*D3DX_PI/2;
			X=CX+rx[c]-1.0f/6+cosf(f)/2;
			Y=CY+ry[c]-1.0f/6+sinf(f)/2;
		} else {
			float f=(float)Time/CONNECTED_RAIL_TIME-0.5f;
			X=CX+1.0f/3+CONNECTED_RAIL_VX[Dir]*f;
			Y=CY+1.0f/3+CONNECTED_RAIL_VY[Dir]*f;
		}
		return true;
	}
	
	virtual void Draw() {}
};

// 
class CConnectedRailStage : public CStage {
public:
	CCell* Cell;
	CCell* PathCell;
	int CX, CY, VX, VY, State;
	CConnectedRailMan* Man;

	CConnectedRailStage() : CStage(L"CONNECTED RAIL") {
		Cell=new CCell(6, 4);
		PathCell=new CCell(Cell->GetXSize(), Cell->GetYSize());
	}
	virtual ~CConnectedRailStage() {
		delete Cell;
		delete PathCell;
	}

	virtual void Init() {
		Cell->Init(
			"152152"
			"457366"
			"157213"
			"453430"
		);
		CX=Cell->GetXSize()-1;
		CY=Cell->GetYSize()-1;
		State=0;
		Man=new CConnectedRailMan(Cell, 1, 0, 0);
	}

	virtual bool Move(const CInputState* is) {
		if (State==0) {
			if (is->Left && CX>0) {
				VX=-1;
				VY=0;
				Time=0;
				State=1;
			}
			if (is->Right && CX<Cell->GetXSize()-1) {
				VX=1;
				VY=0;
				Time=0;
				State=1;
			}
			if (is->Up && CY>0) {
				VX=0;
				VY=-1;
				Time=0;
				State=1;
			}
			if (is->Down && CY<Cell->GetYSize()-1) {
				VX=0;
				VY=1;
				Time=0;
				State=1;
			}
		}
		if (State==1) {
			Time++;
			if (Time==10) {
				if (Man->CX==CX+VX && Man->CY==CY+VY) {
					Man->CX=CX;
					Man->CY=CY;
				}
				Cell->Swap(CX, CY, CX+VX, CY+VY);
				CX+=VX;
				CY+=VY;
				State=0;
			}
		}
		return true;
	}

	virtual void Draw() {
		PathCell->Clear(0);
		int cx=Man->CX, cy=Man->CY, dir=Man->Dir; 
		while (!(PathCell->Get(cx, cy)&(1<<dir))) {
			PathCell->Set(cx, cy, PathCell->Get(cx, cy)|(1<<dir));
			cx+=CONNECTED_RAIL_VX[dir], 
			cy+=CONNECTED_RAIL_VY[dir];
			if (
				cx<0 || cx>=Cell->GetXSize() ||
				cy<0 || cy>=Cell->GetYSize()
			) break;
			int ndir=CONNECTED_RAIL_DIR[Cell->Get(cx, cy)-'0'][dir];
			if (ndir==4) break;
			dir=ndir;
		}

		int xs=Cell->GetXSize(), ys=Cell->GetYSize();
		float 
			sw=Game->GetGraphics()->GetWidth()/xs,
			sh=Game->GetGraphics()->GetHeight()/(ys+1);

		for (int y=0; y<ys; y++) {
			for (int x=0; x<xs; x++) {
				char c=Cell->Get(x, y)-'0';
				float 
					dx=x, dy=y+1, 
					f=Time*0.1f, vx=VX*f, vy=VY*f;
				if (State==1 && x==CX && y==CY) {
					dx+=vx;
					dy+=vy;
				} else
				if (State==1 && x==CX+VX && y==CY+VY) {
					dx-=vx;
					dy-=vy;
				}
				dx*=sw;
				dy*=sh;
				char pc=PathCell->Get(x, y);
				if (pc) {
					if (c==7 && !(pc&0x05)) {
						Game->Texture[TEX_FILL]->Draw(
							dx+sw/3, dy, sw/3, sh,
							0, 0, 1, 1, COL_LGRAY);
					} else
					if (c==7 && !(pc&0x0a)) {
						Game->Texture[TEX_FILL]->Draw(
							dx, dy+sh/3, sw, sh/3,
							0, 0, 1, 1, COL_LGRAY);
					} else {
						Game->Texture[TEX_FILL]->Draw(
							dx, dy, sw, sh,
							0, 0, 1, 1, COL_LGRAY);
					}
				}
				Game->Texture[TEX_RAIL0+c]->Draw(
					dx, dy, sw, sh,
					0, 0, 1, 1, COL_BLACK);
			}
		}

		float dx=Man->X, dy=Man->Y+1;
		if (State==1 && Man->CX==CX+VX && Man->CY==CY+VY) {
			dx-=VX*Time*0.1f;
			dy-=VY*Time*0.1f;
		}
		Game->Texture[TEX_MAN]->Draw(
			dx*sw, dy*sh, sw/3, sh/3, 0, 0, 1, 1, COL_BLACK);
	}
};

//==============================================================
//  ϱ

#define CONNECTED_PIPE_COUNT 7

// 
class CConnectedPipeStage : public CStage {
	CCell* Cell;
	CCell* PathCell;
	int CX, CY, NewPipe;
	bool PrevLever, PrevButton;
public:
	CConnectedPipeStage() : CStage(L"CONNECTED PIPE") {
		Cell=new CCell();
		PathCell=new CCell();
	}
	virtual ~CConnectedPipeStage() {
		delete Cell;
		delete PathCell;
	}

	virtual void Init() {
		Cell->Init(
			"                "
			"################"
			"#+            +#"
			"#              #"
			"#              #"
			"#              #"
			"#              #"
			"#              #"
			"#              #"
			"#              #"
			"#+            +#"
			"################"
		);
		CX=Cell->GetXSize()/2;
		CY=Cell->GetYSize()/2;
		PrevLever=true;
		PrevButton=true;
		NewPipe=Rand.Int31()%CONNECTED_PIPE_COUNT+1;
	}

	virtual bool Move(const CInputState* is) {
		int cx=CX, cy=CY;
		if (!PrevLever) {
			if (is->Left) cx--; else
			if (is->Right) cx++; else
			if (is->Up) cy--; else
			if (is->Down) cy++;
			char c=Cell->Get(cx, cy);
			if (c!='#' && c!='+') {
				CX=cx;
				CY=cy;
			}
		}
		PrevLever=is->Left|is->Right|is->Up|is->Down;
		if (!PrevButton && is->Button[0]) {
			Cell->Set(cx, cy, '0'+NewPipe);
			NewPipe=Rand.Int31()%CONNECTED_PIPE_COUNT+1;
		}
		PrevButton=is->Button[0];
		return true;
	}

	virtual void Draw() {
		int xs=Cell->GetXSize(), ys=Cell->GetYSize();

		PathCell->Clear(0);
		for (int y=0; y<ys; y++) {
			for (int x=0; x<xs; x++) {
				if (Cell->Get(x, y)=='+') {
					for (int i=0; i<4; i++) {
						int cx=x, cy=y, dir=i;
						while (!(PathCell->Get(cx, cy)&(1<<dir))) {
							PathCell->Set(cx, cy, PathCell->Get(cx, cy)|(1<<dir));
							cx+=CONNECTED_RAIL_VX[dir], 
							cy+=CONNECTED_RAIL_VY[dir];
							char c=Cell->Get(cx, cy)-'0';
							if (c<0 || c>=CONNECTED_PIPE_COUNT+1) break;
							int ndir=CONNECTED_RAIL_DIR[c][dir];
							if (ndir==4) break;
							dir=ndir;
						}
					}
				}
			}
		}
		float 
			sw=Game->GetGraphics()->GetWidth()/xs,
			sh=Game->GetGraphics()->GetHeight()/ys;
		for (int y=0; y<ys; y++) {
			for (int x=0; x<xs; x++) {
				char c=Cell->Get(x, y);
				int t=TEX_VOID;
				if (c=='+') {
					t=TEX_CONVEYOR1;
				} else
				if (c=='#') {
					t=TEX_FLOOR;
				} else
				if ('0'<=c && c<'0'+CONNECTED_PIPE_COUNT+1) {
					t=TEX_RAIL0+c-'0';
					char pc=PathCell->Get(x, y);
					if (pc) {
						if (c=='7' && !(pc&0x05)) {
							Game->Texture[TEX_FILL]->Draw(
								x*sw+sw/3, y*sh, sw/3, sh, 
								0, 0, 1, 1, COL_LGRAY);
						} else
						if (c=='7' && !(pc&0x0a)) {
							Game->Texture[TEX_FILL]->Draw(
								x*sw, y*sh+sh/3, sw, sh/3, 
								0, 0, 1, 1, COL_LGRAY);
						} else {
							Game->Texture[TEX_FILL]->Draw(
								x*sw, y*sh, sw, sh, 
								0, 0, 1, 1, COL_LGRAY);
						}
					}
				}
				Game->Texture[t]->Draw(x*sw, y*sh, sw, sh, 0, 0, 1, 1, COL_BLACK);
			}
		}
		Game->Texture[TEX_RAIL0+NewPipe]->Draw(CX*sw, CY*sh, sw, sh, 0, 0, 1, 1, COL_LGRAY);
	}
};

//==============================================================
// Ͽ  

// 
class CLinkedShapeMolecule : public CMover {
public:
	CLinkedShapeMolecule* Prev;
	CLinkedShapeMolecule* Next;
	float VX, VY;

	CLinkedShapeMolecule(float x, float y) 
	:	Prev(NULL), Next(NULL)
	{
		Texture=Game->Texture[TEX_MOLECULE1];
		X=Rand.Int31()%2*(MAX_X+1)-1;
		Y=Rand.Int31()%2*(MAX_Y+1)-1;
		VX=x-X;
		VY=y-Y;
		float v=sqrtf(VX*VX+VY*VY)/0.02f;
		if (v>0) {
			VX/=v;
			VY/=v;
		}
	}

	virtual bool Move(const CInputState* is) {
		if (!Prev || Prev==this) {
			X+=VX;
			Y+=VY;
			Angle+=0.01f;
			return X>=-1 && X<=MAX_X && Y>=-1 && Y<=MAX_Y;
		}
		return true;
	}
};

// Ŀ
class CLinkedShapeCursor : public CLinkedShapeMolecule {
public:
	bool PrevButton;

	CLinkedShapeCursor() 
	:	CLinkedShapeMolecule(0, 0), PrevButton(true)
	{
		Texture=Game->Texture[TEX_MOLECULE0];
		X=(MAX_X-1)/2;
		Y=(MAX_Y-1)/2;
		Prev=this;
	}

	virtual bool Move(const CInputState* is) {
		static const float speed=0.1f;
		VX=VY=0;
		if (is->Left) VX=-speed;
		if (is->Right) VX=speed;
		if (is->Up) VY=-speed;
		if (is->Down) VY=speed;
		if (X+VX<0 || X+VX>MAX_X-1) VX=0;
		if (Y+VY<0 || Y+VY>MAX_Y-1) VY=0;

		CLinkedShapeMolecule* tail=this;
		while (tail->Next) tail=tail->Next;
		
		for (CLinkedShapeMolecule* m=this; m; m=m->Next) {
			m->X+=VX;
			m->Y+=VY;
			for (CTaskIter i(Game->MoverList); i.HasNext(); ) {
				CLinkedShapeMolecule* n=(CLinkedShapeMolecule*)i.Next();
				static const float hit=1.0f, link=0.6f;
				if (
					!n->Prev &&
					abs(n->X-m->X)<hit && abs(n->Y-m->Y)<hit
				) {
					n->X=m->X+(n->X<m->X?-link:link);
					n->Y=m->Y+(n->Y<m->Y?-link:link);
					n->Angle=0;
					n->Prev=tail;
					tail->Next=n;
					tail=n;
				}
			}
		}
		
		if (!PrevButton && is->Button[0]) {
			if (tail->Prev) {
				float 
					vx=tail->X-X, vy=tail->Y-Y,
					v=sqrtf(vx*vx+vy*vy)/0.12f;
				if (v>0) {
					tail->VX=vx/v;
					tail->VY=vy/v;
				}
				tail->Prev->Next=NULL;
				tail->Prev=tail;
			}
		}
		PrevButton=is->Button[0];
		
		return true;
	}
};

// 
class CLinkedShapeStage : public CStage {
	CLinkedShapeCursor* Cursor;
	int Time;
public:
	CLinkedShapeStage() : CStage(L"LINKED SHAPE") {
	}
	virtual ~CLinkedShapeStage() {
	}
	virtual void Init() {
		Cursor=new CLinkedShapeCursor();
		Time=0;
	}
	virtual bool Move(const CInputState* is) {
		Time++;
		if (Time==60) {
			new CLinkedShapeMolecule(Cursor->X, Cursor->Y);
			Time=0;
		}
		return true;
	}
};

//==============================================================
//  ѷ ĥϱ

// 
class CEnclosedAreaEnemy : public CMover {
public:
	CCell* Cell;
	int CX, CY, VX, VY;

	CEnclosedAreaEnemy(CCell* cell) 
	: Cell(cell) 
	{
		CX=Cell->GetXSize()/2;
		CY=3;
		VX=Rand.Int31()%2*2-1;
		VY=Rand.Int31()%2*2-1;
	}

	virtual bool Move(const CInputState* is) {
		if (Cell->Get(CX+VX, CY+VY)!=' ') {
			VX=Rand.Int31()%2*2-1;
			VY=Rand.Int31()%2*2-1;
		} else {
			CX+=VX;
			CY+=VY;
		}
		return true;
	}
	
	virtual void Draw() {}
};

// Ŀ
class CEnclosedAreaCursor : public CMover {
public:
	CCell* Cell;
	CEnclosedAreaEnemy* Enemy;
	int CX, CY;

	CEnclosedAreaCursor(CCell* cell, CEnclosedAreaEnemy* enemy) 
	:	Cell(cell), Enemy(enemy)
	{
		CX=Cell->GetXSize()/2;
		CY=Cell->GetYSize()-3;
	}

	virtual bool Move(const CInputState* is) {
		int vx=0, vy=0;
		if (is->Left) vx=-1; else
		if (is->Right) vx=1; else
		if (is->Up) vy=-1; else
		if (is->Down) vy=1;
		char 
			c1=Cell->Get(CX+vx, CY+vy),
			c2=Cell->Get(CX+vx*2, CY+vy*2);
		if (is->Button[0]) {
			if (c1==' ' && c2==' ') {
				Cell->Set(CX+vx, CY+vy, '+');
				Cell->Set(CX+vx*2, CY+vy*2, '+');
			} else 
			if (c1==' ' && c2=='#') {
				Cell->Set(CX+vx, CY+vy, '+');
				Paint(Enemy->CX, Enemy->CY);
				for (int y=3; y<Cell->GetYSize()-3; y++) {
					for (int x=3; x<Cell->GetXSize()-3; x++) {
						switch (Cell->Get(x, y)) {
							case ' ': 
								Cell->Set(x, y, '*');
								break;
							case '%': 
								Cell->Set(x, y, ' ');
								break;
							case '+':
								Cell->Set(x, y, '#');
								break;
						}
					}
				}
				for (int y=2; y<Cell->GetYSize()-2; y++) {
					for (int x=2; x<Cell->GetXSize()-2; x++) {
						if (Cell->Get(x, y)=='#') {
							int count=0;
							for (int i=-1; i<=1; i++) {
								for (int j=-1; j<=1; j++) {
									if (Cell->Get(x+i, y+j)==' ') {
										count++;
									}
								}
							}
							if (count==0) {
								Cell->Set(x, y, '*');
							}
						}
					}
				}
			} else {
				vx=vy=0;
			}
		} else {
			if (c1!='#' || c2!='#') {
				vx=vy=0;
			}
		}
		CX+=vx*2;
		CY+=vy*2;
		return true;
	}
	
	void Paint(int x, int y) {
		if (Cell->Get(x, y)==' ') {
			int lx, rx;
			for (lx=x; Cell->Get(lx-1, y)==' '; lx--) ;
			for (rx=x; Cell->Get(rx+1, y)==' '; rx++) ;
			for (x=lx; x<=rx; x++) {
				Cell->Set(x, y, '%');
			}
			for (x=lx; x<=rx; x++) {
				Paint(x, y-1);
				Paint(x, y+1);
			}
		}
	}
	
	virtual void Draw() {}
};

// 
class CEnclosedAreaStage : public CStage {
	CCell* Cell;
	CEnclosedAreaEnemy* Enemy;
	CEnclosedAreaCursor* Cursor;
public:
	CEnclosedAreaStage() : CStage(L"ENCLOSED AREA") {
		Cell=new CCell(201, 141);
	}
	virtual ~CEnclosedAreaStage() {
		delete Cell;
	}
	virtual void Init() {
		int xs=Cell->GetXSize(), ys=Cell->GetYSize();
		for (int y=0; y<ys; y++) {
			for (int x=0; x<xs; x++) {
				if (x<2 || x>xs-3 || y<2 || y>ys-3) {
					Cell->Set(x, y, '=');
				} else
				if (x==2 || x==xs-3 || y==2 || y==ys-3) {
					Cell->Set(x, y, '#');
				} else {
					Cell->Set(x, y, ' ');
				}
			}
		}
		Enemy=new CEnclosedAreaEnemy(Cell);
		Cursor=new CEnclosedAreaCursor(Cell, Enemy);
	}
	virtual void Draw() {
		static const int dy=10;
		int xs=Cell->GetXSize(), ys=Cell->GetYSize();
		float 
			sw=Game->GetGraphics()->GetWidth()/(xs-1),
			sh=Game->GetGraphics()->GetHeight()/(ys-1);
		for (int y=0; y<ys; y++) {
			int lx, rx;
			for (lx=rx=0; lx<xs; lx=rx) {
				D3DCOLOR color=COL_WHITE;
				char c=Cell->Get(lx, y);
				switch (c) {
					case '#': color=COL_BLACK; break;
					case '+': color=COL_MGRAY; break;
					case '*': color=COL_LGRAY; break;
				}
				for (rx=lx+1; rx<xs; rx++) {
					if (c!=Cell->Get(rx, y)) {
						Game->Texture[TEX_FILL]->Draw(
							lx*sw, (y+dy)*sh, (rx-lx)*sw, sh, 
							0, 0, 1, 1, color);
						break;
					}
				}
			}
		}
		Game->Texture[TEX_SHURIKEN]->Draw(
			(Cursor->CX-5)*sw, (Cursor->CY-5+dy)*sh, sw*11, sh*11, 
			0, 0, 1, 1, COL_BLACK);
		Game->Texture[TEX_ENEMY1]->Draw(
			(Enemy->CX-10)*sw, (Enemy->CY-10+dy)*sh, sw*21, sh*21, 
			0, 0, 1, 1, COL_BLACK);
	}
};

//==============================================================
//   ׸  ȸϱ

// 
class CTraversableRouteMan : public CMover {
public:
	CCell* Cell;
	int CX, CY, VX, VY;
	int State, Time;

	CTraversableRouteMan(CCell* cell, int x, int y)
	:	Cell(cell), CX(x), CY(y), VX(0), VY(0), State(0), Time(0)
	{
		Texture=Game->Texture[TEX_MAN];
		X=x;
		Y=y;
	}

	virtual bool Move(const CInputState* is) {
		if (State==0) {
			VX=VY=0;
			if (is->Left) VX=-1; else
			if (is->Right) VX=1; else
			if (is->Up) VY=-1; else
			if (is->Down) VY=1;
			
			if (VX!=0 || VY!=0) {
				char c=Cell->Get(CX+VX, CY+VY);
				if (c==' ' || c=='I') {
					Cell->Set(CX, CY, '+');
					CX+=VX;
					CY+=VY;
					Cell->Set(CX, CY, ' ');
					State=1;
					Time=0;
				}
			}
		}
		if (State==1) {
			Time++;
			X=CX-VX*(1-Time*0.1f);
			Y=CY-VY*(1-Time*0.1f);
			if (Time==10) {
				State=0;
			}
		}
		return true;
	}
};

// 
class CTraversableRouteStage : public CStage {
	CCell* Cell;
public:
	CTraversableRouteStage() : CStage(L"TRAVERSABLE ROUTE") {
		Cell=new CCell();
	}
	virtual ~CTraversableRouteStage() {
		delete Cell;
	}
	virtual void Init() {
		Cell->Init(
			"                "
			"################"
			"#I  # I#   I   #"
			"# # #  # # ### #"
			"# #I   # #    I#"
			"# # ## M #I# ###"
			"# I  # ### # I #"
			"#### #  I    # #"
			"#   I# # ## ## #"
			"# ## #I# #I  # #"
			"#I     #   #  I#"
			"################"
		);
		for (int y=0; y<MAX_Y; y++) {
			for (int x=0; x<MAX_X; x++) {
				if (Cell->Get(x, y)=='M') {
					Cell->Set(x, y, ' ');
					new CTraversableRouteMan(Cell, x, y);
				}
			}
		}
	}
	virtual void Draw() {
		float 
			sw=Game->GetGraphics()->GetWidth()/MAX_X,
			sh=Game->GetGraphics()->GetHeight()/MAX_Y;
		for (int y=0; y<MAX_Y; y++) {
			for (int x=0; x<MAX_X; x++) {
				switch (Cell->Get(x, y)) {
					case '#':
						Game->Texture[TEX_FLOOR]->Draw(
							x*sw, y*sh, sw, sh, 0, 0, 1, 1, COL_BLACK
						);
						break;
					case 'I':
						Game->Texture[TEX_MUSHROOM]->Draw(
							x*sw, y*sh, sw, sh, 0, 0, 1, 1, COL_BLACK
						);
						break;
					case '+':
						Game->Texture[TEX_FILL]->Draw(
							x*sw, y*sh, sw, sh, 0, 0, 1, 1, COL_LGRAY
						);
						break;
				}
			
			}
		}
	}
};

//==============================================================
// ܾ 

#define CROSSWORD_KANA_COUNT 5
#define CROSSWORD_MIN_LENGTH 2
#define CROSSWORD_MAX_LENGTH 4

// 
class CCrosswordStage : public CStage {
public:
	CCell* Cell;
	int CX, CY, Kana, State;
	int X, Y, VX, VY, Index, Length, Time;
	bool PrevLever, PrevButton;

	CCrosswordStage() 
	:	CStage(L"CROSSWORD"), 
		CX(0), CY(0), Kana(0), State(0), 
		PrevLever(true), PrevButton(true)
	{
		Cell=new CCell(12, 8);
	}

	virtual ~CCrosswordStage() {
		delete Cell;
	}

	virtual void Init() {
		Cell->Clear(CROSSWORD_KANA_COUNT);
	}

	virtual bool Move(const CInputState* is) {
		int xs=Cell->GetXSize(), ys=Cell->GetYSize();
		if (State==0) {
			Kana=Rand.Int31()%CROSSWORD_KANA_COUNT;
			State=1;
		}
		if (State==1) {
			if (!PrevLever) {
				if (is->Left && CX>0) CX--; else
				if (is->Right && CX<xs-1) CX++; else
				if (is->Up && CY>0) CY--; else
				if (is->Down && CY<ys-1) CY++;
			}
			PrevLever=is->Left||is->Right||is->Up||is->Down;
			if (
				!PrevButton && 
				is->Button[0] && 
				Cell->Get(CX, CY)==CROSSWORD_KANA_COUNT
			) {
				Cell->Set(CX, CY, Kana);
				X=CX;
				Y=max(0, CY-CROSSWORD_MAX_LENGTH+1);
				VX=0;
				VY=1;
				Length=CROSSWORD_MIN_LENGTH;
				State=2;
			}
			PrevButton=is->Button[0];
		}
		static const wchar_t* dictionary[]={
			L"", L"", L"", L"", L"",
			L"", L"", L"", L"", L"", L"", 
			L"", L"", L"",
			L"", L"",
			L"", L"", L"", L"", L"", 
			NULL
		};
		if (State==2) {
			wchar_t word[CROSSWORD_MAX_LENGTH+1];
			for (; X<=CX && Y<=CY; X+=VX, Y+=VY, Length=CROSSWORD_MIN_LENGTH) {
				int max_length=min(CROSSWORD_MAX_LENGTH, (xs-X)*VX+(ys-Y)*VY);
				for (; Length<=max_length; Length++) {
					int i, x, y;
					for (i=0, x=X, y=Y; i<Length; i++, x+=VX, y+=VY) {
						word[i]=L"?"[Cell->Get(x, y)];
					}
					word[i]=L'\0';
					if (x>CX || y>CY) {
						for (i=0; dictionary[i]; i++) {
							if (wcscmp(word, dictionary[i])==0) break;
						}
						if (dictionary[i]) break;
					}
				}
				if (Length<=max_length) break;
			}
			if (Y>CY) {
				X=max(0, CX-CROSSWORD_MAX_LENGTH+1);
				Y=CY;
				VX=1;
				VY=0;
				Length=CROSSWORD_MIN_LENGTH;
			} else
			if (X>CX) {
				State=0;
			} else {
				for (int i=0, x=X, y=Y; i<Length; i++, x+=VX, y+=VY) {
					Cell->Set(x, y, Cell->Get(x, y)|0x80);
				}
				Length++;
				Time=0;
				State=3;
			}
		}
		if (State==3) {
			Time++;
			if (Time==30) {
				for (int i=0, x=X, y=Y; i<Length; i++, x+=VX, y+=VY) {
					Cell->Set(x, y, Cell->Get(x, y)&0x7f);
				}
				State=2;
			}
		}
		return true;
	}

	virtual void Draw() {
		int xs=Cell->GetXSize(), ys=Cell->GetYSize();
		float 
			sw=Game->GetGraphics()->GetWidth()/xs,
			sh=Game->GetGraphics()->GetHeight()/(ys+1);
		for (int y=0; y<ys; y++) {
			for (int x=0; x<xs; x++) {
				char c=Cell->Get(x, y);
				D3DCOLOR color=(c&0x80)?COL_LGRAY:COL_BLACK;
				c=(c&0x3f);
				if (0<=c && c<CROSSWORD_KANA_COUNT) {
					Game->Texture[TEX_KANA0+c]->Draw(
						x*sw, (y+1)*sh, sw, sh, 0, 0, 1, 1, color);
				}
			}
		}
		if (State<2) {
			Game->Texture[TEX_KANA0+Kana]->Draw(
				CX*sw, (CY+1)*sh, sw, sh, 0, 0, 1, 1, COL_MGRAY);
		}
	}
};




